@@ -6,14 +6,13 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '1.015';
+our $VERSION = '1.016';
 
 use Cwd              qw( realpath );
 use Encode           qw( encode decode );
 use File::Temp       qw( tempfile );
 use List::Util       qw( all );
 
-#use Clone                  qw( clone );
 use List::MoreUtils        qw( first_index );
 use Encode::Locale         qw();
 #use Spreadsheet::Read      qw( ReadData rows ); # "require"d
@@ -52,9 +51,7 @@ sub __insert_into {
             { prompt => 'Columns:', index => 1, no_spacebar => [ 0 .. $#pre ] }
         );
         if ( ! defined $idx[0] || ! defined $choices->[$idx[0]] ) {
-            if ( ! @{$sql->{quote}{chosen_cols}} ) {
-                return;
-            }
+            return if ! @{$sql->{quote}{chosen_cols}};
             $sql->{quote}{chosen_cols} = [];
             $sql->{print}{chosen_cols} = [];
             @cols = ( @$pr_columns );
@@ -164,28 +161,22 @@ sub __insert_into {
             }
         }
         else {
-            if ( $input_mode eq 'Paste' ) {
+            if ( $input_mode eq 'Multi-row' ) {
                 my ( $fh, $file ) = tempfile( DIR => $self->{info}{app_dir}, UNLINK => 1 , SUFFIX => '.csv' );
-                binmode $fh, ':encoding(' . $self->{opt}{insert}{file_encoding} . ')';
+                binmode $fh, ':encoding(' . $self->{opt}{insert}{file_encoding} . ')' or die $!;
                 $auxil->__print_sql_statement( $sql, $table, $sql_type );
-                print 'Multirow: ' . "\n";
+                print 'Multi row: ' . "\n";
                 # STDIN
-                print $fh do { local $/; <STDIN> };
+                while ( my $row = <STDIN> ) {
+                    print $fh $row;
+                }
                 seek $fh, 0, 0;
-                if ( $self->{opt}{insert}{parse_mode} < 2 ) {
-                    ( $sql->{quote}{insert_into_args} ) = $self->__parse_file( $fh );
-                    if ( ! @{$sql->{quote}{insert_into_args}} ) {
-                        my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
-                        $cm->choose( [ 'empty file!' ] );
-                    }
-                    $self->__filter_input( $sql, $table, $sql_type, $fh );
-                }
-                else {
-                    my $sheet_idx;
-                    ( $sql->{quote}{insert_into_args}, $sheet_idx ) = $self->__parse_file( $file );
-                    $self->__filter_input( $sql, $table, $sql_type, $file, $sheet_idx );
-                }
+                $self->__get_insert_into_args( $sql, $table, $sql_type, $fh );
                 if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                    #if ( $self->{opt}{insert}{parse_mode} < 2 ) {
+                    #     my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
+                    #    $cm->choose( [ 'empty file!' ] );
+                    #}
                     if ( @{$self->{opt}{insert}{input_modes}} == 1 ) {
                         $sql->{quote}{chosen_cols} = [];
                         $sql->{print}{chosen_cols} = [];
@@ -199,7 +190,7 @@ sub __insert_into {
                 FILE: while ( 1 ) {
                     my @files;
                     if ( $self->{opt}{insert}{max_files} && -e $self->{info}{input_files} ) {
-                        open my $fh_in, '<', $self->{info}{input_files} or die $!;
+                        open my $fh_in, '<:encoding(locale_fs)', $self->{info}{input_files} or die $!;
                         while ( my $fl = <$fh_in> ) {
                             chomp $fl;
                             next if ! -e $fl;
@@ -210,9 +201,7 @@ sub __insert_into {
                     my @files_sorted = sort map { decode 'locale_fs', $_ } @files;
                     if ( length $file ) {
                         my $i = first_index { decode( 'locale_fs', $file ) eq $_ } @files_sorted;
-                        if ( $i > -1  ) {
-                            splice @files_sorted, $i, 1;
-                        }
+                        splice @files_sorted, $i, 1 if $i > -1;
                         unshift @files_sorted, decode 'locale_fs', $file;
                     }
                     my $add_file = 'New file';
@@ -248,14 +237,12 @@ sub __insert_into {
                         $file = realpath encode 'locale_fs', $file;
                         if ( $self->{opt}{insert}{max_files} ) {
                             my $i = first_index { $file eq $_ } @files;
-                            if ( $i > -1  ) {
-                                splice @files, $i, 1;
-                            }
+                            splice @files, $i, 1 if $i > -1;
                             push @files, $file;
                             while ( @files > $self->{opt}{insert}{max_files} ) {
                                 shift @files;
                             }
-                            open my $fh_out, '>', $self->{info}{input_files} or die $!;
+                            open my $fh_out, '>:encoding(locale_fs)', $self->{info}{input_files} or die $!;
                             for my $fl ( @files ) {
                                 print $fh_out $fl . "\n";
                             }
@@ -265,20 +252,20 @@ sub __insert_into {
                     else {
                         $file = realpath encode 'locale_fs', $file;
                     }
+
                     if ( $self->{opt}{insert}{parse_mode} < 2 && -T $file ) {
                         open my $fh, '<:encoding(' . $self->{opt}{insert}{file_encoding} . ')', $file or die $!;
-                        ( $sql->{quote}{insert_into_args} ) = $self->__parse_file( $fh );
-                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                        if ( -z $file ) {
                             my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
                             $cm->choose( [ 'empty file!' ] );
+                            close $fh;
+                            next FILE;
                         }
-                        $self->__filter_input( $sql, $table, $sql_type, $fh );
+                        $self->__get_insert_into_args( $sql, $table, $sql_type, $fh );
                         close $fh;
                     }
                     else {
-                        my $sheet_idx;
-                        ( $sql->{quote}{insert_into_args}, $sheet_idx ) = $self->__parse_file( $file );
-                        $self->__filter_input( $sql, $table, $sql_type, $file, $sheet_idx );
+                        $self->__get_insert_into_args( $sql, $table, $sql_type, $file );
                     }
                     next FILE if ! @{$sql->{quote}{insert_into_args}};
                     last FILE;
@@ -290,8 +277,159 @@ sub __insert_into {
 }
 
 
+sub __get_insert_into_args {
+    my ( $self, $sql, $table, $sql_type, $file_or_fh ) = @_;
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
+
+    SHEET: while ( 1 ) {
+        my ( $nr_of_sheets, $sheet_idx ) = $self->__parse_file( $sql, $table, $sql_type, $file_or_fh );
+        last SHEET if ! @{$sql->{quote}{insert_into_args}};
+        my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
+
+        FILTER: while ( 1 ) {
+            my @pre = ( undef, $self->{info}{ok} );
+            my ( $input_cols, $input_rows_choose, $input_rows_range, $cols_to_rows, $reset ) = ( 'Columns', 'Rows-choose', 'Rows-range', 'Cols_to_Rows', 'Reset' );
+            my $choices = [ @pre, $input_cols, $input_rows_choose, $input_rows_range, $cols_to_rows, $reset ];
+            $auxil->__print_sql_statement( $sql, $table, $sql_type );
+            # Choose
+            my $filter = $stmt_h->choose(
+                $choices,
+                { prompt => 'Filter:' }
+            );
+            if ( ! defined $filter ) {
+                $sql->{quote}{insert_into_args} = [];
+                last SHEET if ! $nr_of_sheets || $nr_of_sheets < 2;
+                next SHEET;
+            }
+            elsif ( $filter eq $reset ) {
+                $self->__parse_file( $sql, $table, $sql_type, $file_or_fh, $sheet_idx );
+                next FILTER
+            }
+            elsif ( $filter eq $self->{info}{ok} ) {
+                last SHEET;
+            }
+            elsif ( $filter eq $input_cols  ) {
+                my @col_idx;
+
+                COLS: while ( 1 ) {
+                    my @pre = ( $self->{info}{ok} );
+                    my $choices = [ @pre, map { "col_$_" } 1 .. @{$sql->{quote}{insert_into_args}[0]} ];
+                    my $prompt = 'Cols: ';
+                    $prompt .= join ',', map { $_ + 1 } @col_idx if @col_idx;
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    # Choose
+                    my @chosen = $stmt_h->choose(
+                        $choices,
+                        { prompt => $prompt, no_spacebar => [ 0 .. $#pre ] }
+                    );
+                    if ( ! defined $chosen[0] ) {
+                        if ( @col_idx ) {
+                            @col_idx = ();
+                            next COLS;
+                        }
+                        else {
+                            next FILTER;
+                        }
+                    }
+                    if ( $chosen[0] eq $self->{info}{ok} ) {
+                        shift @chosen;
+                        for my $col ( @chosen ) {
+                            $col =~ s/^col_//;
+                            push @col_idx, $col - 1;
+                        }
+                        if ( @col_idx ) {
+                            my $tmp = [];
+                            for my $row ( @{$sql->{quote}{insert_into_args}} ) {
+                                push @$tmp, [ @{$row}[@col_idx] ];
+                            }
+                            $sql->{quote}{insert_into_args} = $tmp;
+                        }
+                        next FILTER;
+                    }
+                    for my $col ( @chosen ) {
+                        $col =~ s/^col_//;
+                        push @col_idx, $col - 1;
+                    }
+                }
+            }
+            elsif ( $filter eq $input_rows_range ) {
+                my $stmt_v = Term::Choose->new( $self->{info}{lyt_stmt_v} );
+                my @pre = ( undef );
+                my $choices;
+                {
+                    no warnings 'uninitialized';
+                    $choices = [ @pre, map { join ',', @$_ } @{$sql->{quote}{insert_into_args}} ];
+                }
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                # Choose
+                my $first_idx = $stmt_v->choose(
+                    $choices,
+                    { prompt => "First row:\n", index => 1, undef => '<<' }
+                );
+                next FILTER if ! defined $first_idx || ! defined $choices->[$first_idx];
+                my $first_row = $first_idx - @pre;
+                next FILTER if $first_row < 0;
+                $choices->[$first_row + @pre] = '* ' . $choices->[$first_row + @pre];
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                # Choose
+                my $last_idx = $stmt_v->choose(
+                    $choices,
+                    { prompt => "Last row:\n", default => $first_row, index => 1, undef => '<<' }
+                );
+                next FILTER if ! defined $last_idx || ! defined $choices->[$last_idx];
+                my $last_row = $last_idx - @pre;
+                next FILTER if $last_row < 0;
+                if ( $last_row < $first_row ) {
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    # Choose
+                    $stmt_h->choose(
+                        [ "Last row [$last_row] is less than First row [$first_row]!" ],
+                        { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' }
+                    );
+                    next FILTER;
+                }
+                $sql->{quote}{insert_into_args} = [ @{$sql->{quote}{insert_into_args}}[$first_row .. $last_row] ];
+                next FILTER;
+            }
+            elsif ( $filter eq $input_rows_choose ) {
+                my $stmt_v = Term::Choose->new( $self->{info}{lyt_stmt_v} );
+                my @pre = ( undef );
+                my $choices;
+                {
+                    no warnings 'uninitialized';
+                    $choices = [ @pre, map { join ',', @$_ } @{$sql->{quote}{insert_into_args}} ];
+                }
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                # Choose
+                my @idx = $stmt_v->choose(
+                    $choices,
+                    { prompt => 'Choose rows:', index => 1, no_spacebar => [ 0 .. $#pre ], undef => '<<' }
+                );
+                next FILTER if ! defined $idx[0] || ! defined $choices->[$idx[0]];
+                my @row_idx = map{ $_ - @pre } @idx;
+                $sql->{quote}{insert_into_args} = [ @{$sql->{quote}{insert_into_args}}[@row_idx] ];
+                next FILTER;
+            }
+            elsif ( $filter eq $cols_to_rows ) {
+                my $tmp = $sql->{quote}{insert_into_args};
+                my $new = [];
+                for my $i ( 0 .. $#$tmp ) {
+                    for my $j ( 0 .. $#{$tmp->[$i]} ) {
+                        $new->[$j][$i] = $tmp->[$i][$j];
+                    }
+                }
+                $sql->{quote}{insert_into_args} = $new;
+                next FILTER;
+            }
+        }
+    }
+}
+
+
 sub __parse_file {
-    my ( $self, $file_or_fh, $sheet_idx ) = @_;
+    my ( $self, $sql, $table, $sql_type, $file_or_fh, $sheet_idx ) = @_;
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
+    $auxil->__print_sql_statement( $sql, $table, $sql_type );
     if ( ref $file_or_fh eq 'GLOB' ) {
         my $fh = $file_or_fh;
         seek $fh, 0, 0;
@@ -306,7 +444,8 @@ sub __parse_file {
             local $/;
             push @$tmp, map { [ split /$self->{opt}{insert}{i_f_s}/ ] } split /$self->{opt}{insert}{i_r_s}/, <$fh>;
         }
-        return $tmp;
+        $sql->{quote}{insert_into_args} = $tmp;
+        return;
     }
     else {
         my $file = $file_or_fh;
@@ -314,25 +453,26 @@ sub __parse_file {
         my $file_dc = decode( 'locale_fs', $file );
         if ( ! -e $file ) {
             $cm->choose( [ $file_dc . ' : file not found!' ] );
-            return [];
+            return;
         }
         if ( ! -s $file ) {
-            $cm->choose( [ $file_dc . ' : file is empty!' ] );
+            $cm->choose( [ $file_dc . ' : empty file!' ] );
             return;
         }
         if ( ! -r $file ) {
-            $cm->choose( [ $file_dc . ' : file is not readable!' ] );
-            return [];
+            $cm->choose( [ $file_dc . ' : file not readable!' ] );
+            return;
         }
         require Spreadsheet::Read;
         my $book = Spreadsheet::Read::ReadData( $file, cells => 0, attr => 0, rc => 1, strip => 0 );
         if ( ! defined $book ) {
             $cm->choose( [ $file_dc . ' : no book!' ] );
-            return [];
+            return;
         }
-        elsif ( @$book < 2 ) { # first sheet in $book contains meta info
+        my $nr_of_sheets = @$book - 1; # first sheet in $book contains meta info
+        if ( $nr_of_sheets == 0 ) {
             $cm->choose( [ $file_dc . ' : no sheets!' ] );
-            return [];
+            return;
         }
         if ( $sheet_idx ) {
             $sheet_idx = $sheet_idx;
@@ -351,172 +491,17 @@ sub __parse_file {
                 { %{$self->{info}{lyt_stmt_v}}, index => 1, prompt => 'Choose a sheet' }
             );
             if ( ! defined $sheet_idx || ! defined $choices->[$sheet_idx] ) {
-                return [];
+                return;
             }
         }
         if ( $book->[$sheet_idx]{maxrow} == 0 ) {
             my $cm = Term::Choose->new( { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' } );
             my $sheet = length $book->[$sheet_idx]{label} ? $book->[$sheet_idx]{label} : 'sheet_' . $_;
             $cm->choose( [ $sheet . ': empty sheet!' ] );
-            return [];
-        }
-        return [ Spreadsheet::Read::rows( $book->[$sheet_idx] ) ], $sheet_idx;
-    }
-}
-
-
-sub __filter_input {
-    my ( $self, $sql, $table, $sql_type, $file_or_fh, $sheet_idx ) = @_;
-    return if ! @{$sql->{quote}{insert_into_args}};
-    my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
-    my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
-    #my $backup = clone $sql->{quote}{insert_into_args};
-
-    FILTER: while ( 1 ) {
-        my @pre = ( undef, $self->{info}{ok} );
-        my ( $input_cols, $input_rows_choose, $input_rows_range, $cols_to_rows, $reset ) = ( 'Columns', 'Rows-choose', 'Rows-range', 'Cols_to_Rows', 'Reset' );
-        my $choices = [ @pre, $input_cols, $input_rows_choose, $input_rows_range, $cols_to_rows, $reset ];
-        $auxil->__print_sql_statement( $sql, $table, $sql_type );
-        # Choose
-        my $choice = $stmt_h->choose(
-            $choices,
-            { prompt => 'Filter:' }
-        );
-        if ( ! defined $choice ) {
-            $sql->{quote}{insert_into_args} = [];
             return;
         }
-        elsif ( $choice eq $reset ) {
-            ( $sql->{quote}{insert_into_args} ) = $self->__parse_file( $file_or_fh, $sheet_idx );
-            #$sql->{quote}{insert_into_args} = clone $backup;
-            next FILTER
-        }
-        elsif ( $choice eq $self->{info}{ok} ) {
-            return;
-        }
-        elsif ( $choice eq $input_cols  ) {
-            my @col_idx;
-
-            COLS: while ( 1 ) {
-                my @pre = ( $self->{info}{ok} );
-                my $choices = [ @pre, map { "col_$_" } 1 .. @{$sql->{quote}{insert_into_args}[0]} ];
-                my $prompt = 'Cols: ';
-                $prompt .= join ',', map { $_ + 1 } @col_idx if @col_idx;
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
-                # Choose
-                my @chosen = $stmt_h->choose(
-                    $choices,
-                    { prompt => $prompt, no_spacebar => [ 0 .. $#pre ] }
-                );
-                if ( ! defined $chosen[0] ) {
-                    if ( @col_idx ) {
-                        @col_idx = ();
-                        next COLS;
-                    }
-                    else {
-                        next FILTER;
-                    }
-                }
-                if ( $chosen[0] eq $self->{info}{ok} ) {
-                    shift @chosen;
-                    for my $col ( @chosen ) {
-                        $col =~ s/^col_//;
-                        push @col_idx, $col - 1;
-                    }
-                    if ( @col_idx ) {
-                        my $tmp = [];
-                        for my $row ( @{$sql->{quote}{insert_into_args}} ) {
-                            push @$tmp, [ @{$row}[@col_idx] ];
-                        }
-                        $sql->{quote}{insert_into_args} = $tmp;
-                    }
-                    next FILTER;
-                }
-                for my $col ( @chosen ) {
-                    $col =~ s/^col_//;
-                    push @col_idx, $col - 1;
-                }
-            }
-        }
-        elsif ( $choice eq $input_rows_range ) {
-            my $stmt_v = Term::Choose->new( $self->{info}{lyt_stmt_v} );
-            my @pre = ( undef );
-            my $choices;
-            {
-                no warnings 'uninitialized';
-                $choices = [ @pre, map { join ',', @$_ } @{$sql->{quote}{insert_into_args}} ];
-            }
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
-            # Choose
-            my $first_idx = $stmt_v->choose(
-                $choices,
-                { prompt => "First row:\n", index => 1, undef => '<<' }
-            );
-            if ( ! defined $first_idx || ! defined $choices->[$first_idx] ) {
-                next FILTER;
-            }
-            my $first_row = $first_idx - @pre;
-            if ( $first_row < 0 ) {
-                next FILTER;
-            }
-            $choices->[$first_row + @pre] = '* ' . $choices->[$first_row + @pre];
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
-            # Choose
-            my $last_idx = $stmt_v->choose(
-                $choices,
-                { prompt => "Last row:\n", default => $first_row, index => 1, undef => '<<' }
-            );
-            if ( ! defined $last_idx || ! defined $choices->[$last_idx] ) {
-                next FILTER;
-            }
-            my $last_row = $last_idx - @pre;
-            if ( $last_row < 0 ) {
-                next FILTER;
-            }
-            if ( $last_row < $first_row ) {
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
-                # Choose
-                $stmt_h->choose(
-                    [ "Last row [$last_row] is less than First row [$first_row]!" ],
-                    { %{$self->{info}{lyt_stop}}, prompt => 'Press ENTER' }
-                );
-                next FILTER;
-            }
-            $sql->{quote}{insert_into_args} = [ @{$sql->{quote}{insert_into_args}}[$first_row .. $last_row] ];
-            next FILTER;
-        }
-        elsif ( $choice eq $input_rows_choose ) {
-            my $stmt_v = Term::Choose->new( $self->{info}{lyt_stmt_v} );
-            my @pre = ( undef );
-            my $choices;
-            {
-                no warnings 'uninitialized';
-                $choices = [ @pre, map { join ',', @$_ } @{$sql->{quote}{insert_into_args}} ];
-            }
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
-            # Choose
-            my @idx = $stmt_v->choose(
-                $choices,
-                { prompt => 'Choose rows:', index => 1, no_spacebar => [ 0 .. $#pre ], undef => '<<' }
-            );
-            if ( ! defined $idx[0] || ! defined $choices->[$idx[0]] ) {
-                next FILTER;
-            }
-            my @row_idx = map{ $_ - @pre } @idx;
-            $sql->{quote}{insert_into_args} = [ @{$sql->{quote}{insert_into_args}}[@row_idx] ];
-            next FILTER;
-        }
-        elsif ( $choice eq $cols_to_rows ) {
-            my $tmp = $sql->{quote}{insert_into_args};
-            my $new = [];
-            for my $i ( 0 .. $#$tmp ) {
-                for my $j ( 0 .. $#{$tmp->[$i]} ) {
-                    $new->[$j][$i] = $tmp->[$i][$j];
-                }
-            }
-            $sql->{quote}{insert_into_args} = $new;
-            next FILTER;
-        }
+        $sql->{quote}{insert_into_args} = [ Spreadsheet::Read::rows( $book->[$sheet_idx] ) ];
+        return $nr_of_sheets, $sheet_idx;
     }
 }
 
